# This is code to replicate the analyses from the American naturalist paper:
# Unprecedented biting perfomance in herbivorous fish: how the complex biting system of Pomacentridae circumvents performance trade-offs

#The R version 3.6.3 was used.

# Packages used in this study:
  #Dataset building
  library(readxl)            #version 1.3.1
  library(dplyr)             #version 1.0.0
  library(tidyr)             #version 1.1.0
  
  #Phylogenetic analyses
  library(ape)               #version 5.3
  library(nlme)              #version 3.1.144
  library(fishtree)          #version 0.3.2
  library(geiger)            #version 2.0.6.4
  library(phytools)          #version 0.7.20


# We used the R code from the function phylANOVA.intra (Lpez-Fernndez et al. 2014).
  source("phylANOVA.intra2.R")
# This source can be downloaded from:
# Lpez-Fernndez, H., J. Arbour, S. Willis, C. Watkins, R. L. Honeycutt, and K. O. Winemiller. 2014. 
# Morphology and efficiency of a specialized foraging behavior, sediment sifting, in neotropical cichlid fishes. 
# PloS ONE 9:e89832.

# data used in this study:

  model_data<-read_excel("Appendix_A.xlsx",sheet=1,skip=1)
  kinematic_data<-read_excel("Appendix_A.xlsx",sheet=2,skip=1,n_max=31) 
  bite.velocity.actino_data<-read_excel("Appendix_A.xlsx",sheet=3,skip=2,n_max=41)
  MA.closing.actino_data<-read_excel("Appendix_A.xlsx",sheet=4,skip=2,n_max=40)

# I) Biomechanical model dataset

# Comparison between (a) the grazers and the non-grazers,
# (b) the cmd species and the non-cmd species

# We first compared the head lengths between the groups.
  HL.grazers.vs.non_grazers<-lm(model_data$`Head_Length_(mm)`~model_data$Group_1)
  HL.cmd.vs.non_cmd<-lm(model_data$`Head_Length_(mm)`~model_data$Group_2)       

# We then compared the biomechanical model inputs between the groups
# with phylogenetic-corrected ANOVAs. 
# The file has to be transformed into a ".txt" or "csv" file
# to run the phylANOVA.intra function.
  write.table(model_data,file="model_data2.txt",sep="\t")
  model_data2<-read.table("model_data2.txt",header=TRUE)

# Create the MA_A2 and MA_A3 variables
  model_data2$MA_A2<-model_data2$Inlever_A2/model_data2$Lower_Jaw_length_.mm.
  model_data2$MA_A3<-model_data2$Inlever_A3/model_data2$Lower_Jaw_length_.mm.

# Note: the following variables were log-transformed to
# fit normal condition.

  model_data2$LOG_Bite_Force<-log(model_data2$Bite_Force_.N.)
  model_data2$LOG_Bite_Speed<-log(model_data2$Lower_jaw_closing_speed_.cm.s.)
  model_data2$LOG_length_A2<-log(model_data2$A2_length)
  model_data2$LOG_length_A3<-log(model_data2$A3_length)
  model_data2$LOG_fibers_A2<-log(model_data2$A2_fibers_length)
  model_data2$LOG_fibers_A3<-log(model_data2$A3_fibers_length)
  model_data2$LOG_PCSA_A2<-log(model_data2$PCSA_A2)
  model_data2$LOG_PCSA_A3<-log(model_data2$PCSA_A3)
 
# The phylANOVA.intra needs two covariates. As we do not run two-way ANOVA
# in this study, we create a covariate with null variance.
  model_data2$null.variance<-rep(1,nrow(model_data2))

# As no phylogenetic information was available for Neopomacentrus fuliginosus,
# we used the phylogenetic information from Neopomacentrus nemurus for the time-tree.
# Pay attention that names in the phylognetic and phenotypic dataset need to correspond.
  model_data2$Species<-dplyr::recode(model_data2$Species,
                                  "Neopomacentrus_fuliginosus"="Neopomacentrus_nemurus")

# read tree
  tree <-read.tree("Phylo_29species.tree")

# run the phylogenetic anova. Pay attention that covariates have to be identified
# by their column number.
  ANOVA.phyl<-phylANOVA.intra(tree,X=model_data2,
                            ycol="dependent variable (column number)",
                            f1col="Group(column number)",
                            f2col="null.variance (column number)",
                            nsim=10000,method="anova")
  
# The "lm" function can be used to get the estimates and standard error for each variable.
  lm.model<-lm("dependent variable"~"Group",
               data=model_data2)
  
# II) Kinematic dataset

# Comparison of the mouth closing speed between the B1
# and the modeled speed

# Select the B1 of kinematic data and the LJspeed of the model

  B1.vs.Bmodel<-rbind(
  data.frame(Speed=kinematic_data$`Lower_jaw_closing_speed_(cm/s)`[kinematic_data$Biting_pattern!="B2"],
           Group=rep("B1",20)),
  data.frame(Speed=model_data$`Lower_jaw_closing_speed_(cm/s)`,
           Group=rep("Bmodel",82))
)

  model<-lm(log(Speed)~Group,
          data=B1.vs.Bmodel)

# III) Actinopterygians mouth closing velocity dataset

# Correct the data type
  bite.velocity.actino_data$SL_cm<-as.numeric(bite.velocity.actino_data$SL_cm)
  bite.velocity.actino_data$`Mouth_closing_speed_(cm/s)`<-as.numeric(bite.velocity.actino_data$`Mouth_closing_speed_(cm/s)`)
  bite.velocity.actino_data$`Duration_(ms)`<-as.numeric(bite.velocity.actino_data$`Duration_(ms)`)

# We do not consider the damselfishes for the models
  bite.velocity.actino_data<-bite.velocity.actino_data[bite.velocity.actino_data$Family!="Pomacentridae",]

# The Labridae represent one third of the observations.
# Thus, we selected one species by family to run the model,
# and we make 1000 iterations.

# We do not consider Lentipes concolor as no phylogenetic information
# is available.
  bite.velocity.actino_data<-bite.velocity.actino_data[bite.velocity.actino_data$Species!="Lentipes concolor",]

# a) mouth closing duration vs. proportion of nekton in the food habit

  output.phylo.duration<-matrix(NA,1000,6)
  colnames(output.phylo.duration)<-c("Intercept","Estimate","SE","t-value","p-value","R-squared")
  
  for (i in 1:1000){
  
  #Build a dataset with one species by family
  Antennariidae<-data.frame(bite.velocity.actino_data%>%
                              filter(Family=="Antennariidae"))
  Belonidae<-data.frame(bite.velocity.actino_data%>%
                          filter(Family=="Belonidae"))
  Centrarchidae.temp<-data.frame(bite.velocity.actino_data%>%
                                   filter(Family=="Centrarchidae"))
  Centrarchidae<-as.data.frame(Centrarchidae.temp[sample(1:nrow(Centrarchidae.temp),1),])
  Chaetodontidae<-data.frame(bite.velocity.actino_data%>%
                               filter(Family=="Chaetodontidae"))
  Cichlidae<-data.frame(bite.velocity.actino_data%>%
                          filter(Family=="Cichlidae"))
  Eleotridae<-data.frame(bite.velocity.actino_data%>%
                           filter(Family=="Eleotridae"))
  Embiotocidae.temp<-data.frame(bite.velocity.actino_data%>%
                                  filter(Family=="Embiotocidae"))
  Embiotocidae<-as.data.frame(Embiotocidae.temp[sample(1:nrow(Embiotocidae.temp),1),])
  Fundulidae<-data.frame(bite.velocity.actino_data%>%
                           filter(Family=="Fundulidae"))
  Gobiidae<-data.frame(bite.velocity.actino_data%>%
                         filter(Family=="Gobiidae"))
  Labridae.temp<-data.frame(bite.velocity.actino_data%>%
                              filter(Family=="Labridae"))
  Labridae<-as.data.frame(Labridae.temp[sample(1:nrow(Labridae.temp),1),])
  Lepisosteidae.temp<-data.frame(bite.velocity.actino_data%>%
                                   filter(Family=="Lepisosteidae"))
  Lepisosteidae<-as.data.frame(Lepisosteidae.temp[sample(1:nrow(Lepisosteidae.temp),1),])
  Lutjanidae <-data.frame(bite.velocity.actino_data%>%
                            filter(Family=="Lutjanidae"))
  Osphronemidae <-data.frame(bite.velocity.actino_data%>%
                               filter(Family=="Osphronemidae"))
  Pleuronectidae <-data.frame(bite.velocity.actino_data%>%
                                filter(Family=="Pleuronectidae"))
  Poeciliidae.temp<-data.frame(bite.velocity.actino_data%>%
                                 filter(Family=="Poeciliidae"))
  Poeciliidae<-as.data.frame(Poeciliidae.temp[sample(1:nrow(Poeciliidae.temp),1),])
  Pomacanthidae <-data.frame(bite.velocity.actino_data%>%
                               filter(Family=="Pomacanthidae"))
  Scaridae.temp<-data.frame(bite.velocity.actino_data%>%
                              filter(Family=="Scaridae"))
  Scaridae<-as.data.frame(Scaridae.temp[sample(1:nrow(Scaridae.temp),1),])
  Serranidae <-data.frame(bite.velocity.actino_data%>%
                            filter(Family=="Serranidae"))
  Sphyraenidae <-data.frame(bite.velocity.actino_data%>%
                              filter(Family=="Sphyraenidae"))
  Synanceiidae <-data.frame(bite.velocity.actino_data%>%
                              filter(Family=="Synanceiidae"))
  
  data_set<-rbind(Antennariidae,Belonidae,Centrarchidae,Chaetodontidae,Cichlidae,     
                  Eleotridae,Embiotocidae,Fundulidae,Gobiidae,Labridae,      
                  Lepisosteidae,Lutjanidae,Osphronemidae,Pleuronectidae,Poeciliidae,   
                  Pomacanthidae,Scaridae,Serranidae,Sphyraenidae,Synanceiidae)  
  
# Tree
# Prepare the data to run the PGLS
  data<-data_set[,c(6,9,10)]
  rownames(data)<-data$Species
  data<-data[,-1]
  data<-data%>%
  drop_na()
  
  species<-rownames(data)
  species<-unlist(species)
  
# Build the tree
  tree<-fishtree_phylogeny(species, type="chronogram")
  write.tree(tree,file="tree_temp.tree")
  tree2<-read.tree("tree_temp.tree")
  philo<-ladderize(tree2)
  
# PGLS
  
  bm<-corBrownian(1,philo)
  model.pgls<-gls(log(Duration_.ms.)~Nekton_prop,
                  correlation=bm,
                  data=data)
  model.coef<-as.data.frame(summary(model.pgls)$tTable)
  
  Intercept<-model.coef[1,1]
  Estimate<-model.coef[2,1]
  SE<-model.coef[2,2]
  t_value<-model.coef[2,3]
  p_value<-model.coef[2,4]
  
# To get the R_squared
  Residuals.variance<-(model.pgls$sigma)^2
  Residuals.SS<-Residuals.variance*(nrow(data)-2)
  SS.nekton<-(anova(model.pgls)[2,2])*Residuals.variance
  R_squared<-SS.nekton/(Residuals.SS+SS.nekton)
  
  output.phylo.duration[i,1]<-Intercept
  output.phylo.duration[i,2]<-Estimate
  output.phylo.duration[i,3]<-SE
  output.phylo.duration[i,4]<-t_value
  output.phylo.duration[i,5]<-p_value
  output.phylo.duration[i,6]<-R_squared
}

# Extract the parameters
  output.phylo.duration<-data.frame(output.phylo.duration)
  Estimate.duration<-mean(output.phylo.duration$Estimate)
  SE.duration<-mean(output.phylo.duration$SE)
  t.value.duration<-mean(output.phylo.duration$t.value)
  p.value.duration<-mean(output.phylo.duration$p.value)
  r.squared.duration<-mean(output.phylo.duration$R.squared)

# b) mouth closing linear speed vs. proportion of nekton in the food habit

# First quit the row with NA values for speed.
  bite.velocity.actino_data.2<-as.data.frame(bite.velocity.actino_data%>%
  tidyr::drop_na(`Mouth_closing_speed_(cm/s)`))

# PGLS
  output.phylo.speed<-matrix(NA,1000,6)
  colnames(output.phylo.speed)<-c("Intercept","Estimate","SE","t-value","p-value","R-squared")
  
  for (i in 1:1000){
  
# Build a dataset with one species by family
  
  Belonidae<-data.frame(bite.velocity.actino_data.2%>%
                          filter(Family=="Belonidae"))
  Centrarchidae.temp<-data.frame(bite.velocity.actino_data.2%>%
                                   filter(Family=="Centrarchidae"))
  Centrarchidae<-as.data.frame(Centrarchidae.temp[sample(1:nrow(Centrarchidae.temp),1),])
  Chaetodontidae<-data.frame(bite.velocity.actino_data.2%>%
                               filter(Family=="Chaetodontidae"))
  Cichlidae<-data.frame(bite.velocity.actino_data.2%>%
                          filter(Family=="Cichlidae"))
  Fundulidae<-data.frame(bite.velocity.actino_data.2%>%
                           filter(Family=="Fundulidae"))
  Labridae.temp<-data.frame(bite.velocity.actino_data.2%>%
                              filter(Family=="Labridae"))
  Labridae<-as.data.frame(Labridae.temp[sample(1:nrow(Labridae.temp),1),])
  Lepisosteidae.temp<-data.frame(bite.velocity.actino_data.2%>%
                                   filter(Family=="Lepisosteidae"))
  Lepisosteidae<-as.data.frame(Lepisosteidae.temp[sample(1:nrow(Lepisosteidae.temp),1),])
  Lutjanidae <-data.frame(bite.velocity.actino_data.2%>%
                            filter(Family=="Lutjanidae"))
  Pleuronectidae <-data.frame(bite.velocity.actino_data.2%>%
                                filter(Family=="Pleuronectidae"))
  Poeciliidae.temp<-data.frame(bite.velocity.actino_data.2%>%
                                 filter(Family=="Poeciliidae"))
  Poeciliidae<-as.data.frame(Poeciliidae.temp[sample(1:nrow(Poeciliidae.temp),1),])
  Pomacanthidae <-data.frame(bite.velocity.actino_data.2%>%
                               filter(Family=="Pomacanthidae"))
  Scaridae.temp<-data.frame(bite.velocity.actino_data.2%>%
                              filter(Family=="Scaridae"))
  Scaridae<-as.data.frame(Scaridae.temp[sample(1:nrow(Scaridae.temp),1),])
  Serranidae <-data.frame(bite.velocity.actino_data.2%>%
                            filter(Family=="Serranidae"))
  Sphyraenidae <-data.frame(bite.velocity.actino_data.2%>%
                              filter(Family=="Sphyraenidae"))
  
  data_set<-rbind(Belonidae,Centrarchidae,Chaetodontidae,Cichlidae,     
              Fundulidae,Labridae,      
              Lepisosteidae,Lutjanidae,Pleuronectidae,Poeciliidae,   
              Pomacanthidae,Scaridae,Serranidae,Sphyraenidae)  
  
 
# Prepare the data to run the PGLS  
  data<-data_set[,c(6,7,8,10)]
  rownames(data)<-data$Species
  data<-data[,-1]
  data<-data%>%
    tidyr::drop_na(Nekton_prop)
  
  species<-rownames(data)
  species<-unlist(species)

  # Build the tree
  tree<-fishtree_phylogeny(species, type="chronogram")
  write.tree(tree,file="tree_temp.tree")
  tree2<-read.tree("tree_temp.tree")
  philo<-ladderize(tree2)
  
# PGLS size-corrected
  
  bm<-corBrownian(1,philo)
  model.pgls.1<-gls(log(Mouth_closing_speed_.cm.s.)~log(SL_cm),
                    data=data,correlation=bm)
  
  data$speed.res<-residuals(model.pgls.1)
  
# PGLS nekton
  
  bm<-corBrownian(1,philo)
  model.pgls.2<-gls(speed.res~Nekton_prop,
                    data=data,correlation=bm)
  
  model.coef<-as.data.frame(summary(model.pgls.2)$tTable)
  
  Intercept<-model.coef[1,1]
  Estimate<-model.coef[2,1]
  SE<-model.coef[2,2]
  t_value<-model.coef[2,3]
  p_value<-model.coef[2,4]
  
# To get the R_squared
  Residuals.variance<-(model.pgls.2$sigma)^2
  Residuals.SS<-Residuals.variance*(nrow(data)-2)
  SS.nekton<-(anova(model.pgls.2)[2,2])*Residuals.variance
  R_squared<-SS.nekton/(Residuals.SS+SS.nekton)
  
  output.phylo.speed[i,1]<-Intercept
  output.phylo.speed[i,2]<-Estimate
  output.phylo.speed[i,3]<-SE
  output.phylo.speed[i,4]<-t_value
  output.phylo.speed[i,5]<-p_value
  output.phylo.speed[i,6]<-R_squared
}

# Extract the parameters
  output.phylo.speed<-data.frame(output.phylo.speed)
  Estimate.speed<-mean(output.phylo.speed$Estimate)
  SE.speed<-mean(output.phylo.speed$SE)
  t.value.speed<-mean(output.phylo.speed$t.value)
  p.value.speed<-mean(output.phylo.speed$p.value)
  r_squared.speed<-mean(output.phylo.speed$R.squared)
  
# D) Actinopterygians MA closing dataset

# We do not consider the damselfishes for the models
  MA.closing.actino_data<-MA.closing.actino_data[MA.closing.actino_data$Family!="Pomacentridae",]

# We do not consider Stephanoberyx monae as no phylogenetic information
# is available.
  MA.closing.actino_data<-MA.closing.actino_data[MA.closing.actino_data$Species!="Stephanoberyx_monae",]

# As no phylogenetic information is available for Apteronotus macrolepis,
# we use the phylogenetic information of Apteronotus albifrons.

  MA.closing.actino_data$Species<-dplyr::recode(MA.closing.actino_data$Species,
                                  "Apteronotus_macrolepis"="Apteronotus_albifrons")

# As no phylogenetic information is available for Harpadon nehereus,
# we use the phylogenetic information of Harpadon microchir.

  MA.closing.actino_data$Species<-dplyr::recode(MA.closing.actino_data$Species,
                                           "Harpadon_nehereus"="Harpadon_microchir")

# Correct the variable type
  MA.closing.actino_data$MA_Closing<-as.numeric(MA.closing.actino_data$MA_Closing)

# Prepare the data to run the PGLS  
  data<-as.data.frame(MA.closing.actino_data[,c(5,6,8)])
  rownames(data)<-data$Species
  data<-data[,-1]
  data<-data%>%
    tidyr::drop_na(Hard_items_prop)
  
  species<-rownames(data)
  species<-unlist(species)

  # Build the tree
  tree<-fishtree_phylogeny(species, type="chronogram")
  write.tree(tree,file="tree_temp.tree")
  tree2<-read.tree("tree_temp.tree")
  philo<-ladderize(tree2)
  
# PGLS hard items
  
  bm<-corBrownian(1,philo)
  model.pgls.3<-gls(MA_Closing~Hard_items_prop,
                    data=data,correlation=bm)
  
  summary(model.pgls.3)

# To get the R_squared
  Residuals.variance<-(model.pgls.3$sigma)^2
  Residuals.SS<-Residuals.variance*(nrow(data)-2)
  SS.hard.item<-(anova(model.pgls.3)[2,2])*Residuals.variance
  R_squared<-SS.hard.item/(Residuals.SS+SS.hard.item)
  
